@@ -3,7 +3,7 @@
 #
 #   Variance estimation using block subdivision
 #
-#   $Revision: 1.4 $  $Date: 2010/11/10 10:44:39 $
+#   $Revision: 1.8 $  $Date: 2010/11/21 07:05:12 $
 #
 
 varblock <- function(X, fun=Kest,
@@ -11,24 +11,49 @@ varblock <- function(X, fun=Kest,
                      nx=3, ny=nx) {
   stopifnot(is.ppp(X))
   stopifnot(is.tess(blocks))
-  # divide data into blocks
-  Y <- split(X, blocks)
-  n <- length(Y)
-  if(n <= 1) stop("Need at least 2 blocks")
+  if(is.character(fun)) fun <- get(fun, mode="function")
+  stopifnot(is.function(fun))
   rvalues <- function(z) { with(z, .x) }
-  # apply 'fun' to each block
-  if(any(c("r", "breaks") %in% names(list(...)))) {
-    # r vector specified
-    fX <- fun(X, ...)
-    z <- lapply(Y, fun, ...)
+  canrestrict <- "domain" %in% names(formals(fun))
+  if(!canrestrict) {
+    # divide data into disjoint blocks
+    Y <- split(X, blocks)
+    n <- length(Y)
+    if(n <= 1) stop("Need at least 2 blocks")
+    # apply 'fun' to each block
+    if(any(c("r", "breaks") %in% names(list(...)))) {
+      # r vector specified
+      fX <- fun(X, ...)
+      z <- lapply(Y, fun, ...)
+    } else {
+      # need to ensure compatible fv objects
+      z <- lapply(Y, fun, ...)
+      rmaxes <- unlist(lapply(z, function(x){ max(rvalues(x)) }))
+      smallest <- which.min(rmaxes)
+      r <- rvalues(z[[smallest]])
+      z <- lapply(Y, fun, ..., r=r)
+      fX <- fun(X, ..., r=r)
+    }
   } else {
-    # need to ensure compatible fv objects
-    z <- lapply(Y, fun, ...)
-    rmaxes <- unlist(lapply(z, function(x){ max(rvalues(x)) }))
-    smallest <- which.min(rmaxes)
-    r <- rvalues(z[[smallest]])
-    z <- lapply(Y, fun, ..., r=r)
-    fX <- fun(X, ..., r=r)
+    # use 'domain' argument of 'fun' to compute contributions from each tile
+    B <- tiles(blocks)
+    n <- length(B)
+    dofun <- function(domain, fun, Xpp, ...) {
+      fun(Xpp, ..., domain=domain)
+    }
+    if(any(c("r", "breaks") %in% names(list(...)))) {
+      # r vector specified
+      fX <- fun(X, ...)
+      z <- lapply(B, dofun, ..., fun=fun, Xpp=X)
+    } else {
+      # need to ensure compatible fv objects
+      z <- lapply(B, dofun, ..., fun=fun, Xpp=X)
+      rmaxes <- unlist(lapply(z, function(x){ max(rvalues(x)) }))
+      smallest <- which.min(rmaxes)
+      r <- rvalues(z[[smallest]])
+      z <- lapply(B, dofun, ..., fun=fun, Xpp=X, r=r)
+      fX <- fun(X, ..., r=r)
+    }
   }
   # find columns that are common to all estimates
   zzz <- reconcile.fv(append(list(fX), z))
@@ -43,15 +68,16 @@ varblock <- function(X, fun=Kest,
   # sample variance
   sqdev <- lapply(z, function(x,m){ eval.fv((x-m)^2) }, m=m)
   v <- meanlistfv(sqdev)
-  v <- eval.fv(v/(n-1))
+  v <- eval.fv(v * n/(n-1))
   v <- rebadge.fv(v, new.ylab=ylab, new.fname=fname, new.yexp=yexp)
   # sample standard deviation
   sd <- eval.fv(sqrt(v))
   sd <- rebadge.fv(sd, new.ylab=ylab, new.fname=fname, new.yexp=yexp)
   # upper and lower limits
-  upper <- eval.fv(fX + 2 * sd)
+  sem <- eval.fv(sd/sqrt(n))
+  upper <- eval.fv(fX + 2 * sem)
   upper <- rebadge.fv(upper, new.ylab=ylab, new.fname=fname, new.yexp=yexp)
-  lower <- eval.fv(fX - 2 * sd)
+  lower <- eval.fv(fX - 2 * sem)
   lower <- rebadge.fv(lower, new.ylab=ylab, new.fname=fname, new.yexp=yexp)
   # tack together 
   m <- prefixfv(m, "mean", "sample mean of")
