@@ -11,7 +11,7 @@
 
 /* Storage of parameters and precomputed/auxiliary data */
 
-struct {
+typedef struct MultiStrauss {
   int ntypes;
   double *beta;    /* beta[i]  for i = 0 ... ntypes-1 */
   double *gamma;   /* gamma[i,j] = gamma[i+ntypes*j] for i,j = 0... ntypes-1 */
@@ -27,15 +27,18 @@ struct {
 
 /* initialiser function */
 
-void straussminit(state, model, algo)
+Cdata *straussminit(state, model, algo)
      State state;
      Model model;
      Algor algo;
 {
   int i, j, ntypes, n2, m, hard;
   double g, r, r2, logg;
+  MultiStrauss *multistrauss;
 
-  MultiStrauss.ntypes = ntypes = model.ntypes;
+  multistrauss = (MultiStrauss *) R_alloc(1, sizeof(MultiStrauss));
+
+  multistrauss->ntypes = ntypes = model.ntypes;
   n2 = ntypes * ntypes;
 
 #ifdef DEBUG
@@ -43,21 +46,21 @@ void straussminit(state, model, algo)
 #endif
 
   /* Allocate space for parameters */
-  MultiStrauss.beta     = (double *) R_alloc((size_t) ntypes, sizeof(double));
-  MultiStrauss.gamma    = (double *) R_alloc((size_t) n2, sizeof(double));
-  MultiStrauss.rad      = (double *) R_alloc((size_t) n2, sizeof(double));
+  multistrauss->beta     = (double *) R_alloc((size_t) ntypes, sizeof(double));
+  multistrauss->gamma    = (double *) R_alloc((size_t) n2, sizeof(double));
+  multistrauss->rad      = (double *) R_alloc((size_t) n2, sizeof(double));
 
   /* Allocate space for transformed parameters */
-  MultiStrauss.rad2     = (double *) R_alloc((size_t) n2, sizeof(double));
-  MultiStrauss.loggamma = (double *) R_alloc((size_t) n2, sizeof(double));
-  MultiStrauss.hard     = (int *) R_alloc((size_t) n2, sizeof(int));
+  multistrauss->rad2     = (double *) R_alloc((size_t) n2, sizeof(double));
+  multistrauss->loggamma = (double *) R_alloc((size_t) n2, sizeof(double));
+  multistrauss->hard     = (int *) R_alloc((size_t) n2, sizeof(int));
 
   /* Allocate scratch space for counts of each pair of types */
-  MultiStrauss.kount     = (int *) R_alloc((size_t) n2, sizeof(int));
+  multistrauss->kount     = (int *) R_alloc((size_t) n2, sizeof(int));
 
   /* Copy and process model parameters*/
   for(i = 0; i < ntypes; i++)
-    MultiStrauss.beta[i]   = model.par[i];
+    multistrauss->beta[i]   = model.par[i];
 
   m = ntypes * (ntypes + 1);
 
@@ -68,33 +71,38 @@ void straussminit(state, model, algo)
       r2 = r * r;
       hard = (g < DOUBLE_EPS);
       logg = (hard) ? 0 : log(g);
-      MAT(MultiStrauss.gamma, i, j, ntypes) = g;
-      MAT(MultiStrauss.rad, i, j, ntypes) = r;
-      MAT(MultiStrauss.hard, i, j, ntypes) = hard; 
-      MAT(MultiStrauss.loggamma, i, j, ntypes) = logg;
-      MAT(MultiStrauss.rad2, i, j, ntypes) = r2;
+      MAT(multistrauss->gamma, i, j, ntypes) = g;
+      MAT(multistrauss->rad, i, j, ntypes) = r;
+      MAT(multistrauss->hard, i, j, ntypes) = hard; 
+      MAT(multistrauss->loggamma, i, j, ntypes) = logg;
+      MAT(multistrauss->rad2, i, j, ntypes) = r2;
     }
   }
   /* periodic boundary conditions? */
-  MultiStrauss.period = model.period;
-  MultiStrauss.per    = (model.period[0] > 0.0);
+  multistrauss->period = model.period;
+  multistrauss->per    = (model.period[0] > 0.0);
 
 #ifdef DEBUG
   Rprintf("end initialiser\n");
 #endif
+  return((Cdata *) multistrauss);
 }
 
 /* conditional intensity evaluator */
 
-double straussmcif(prop, state)
+double straussmcif(prop, state, cdata)
      Propo prop;
      State state;
+     Cdata *cdata;
 {
   int npts, ntypes, kount, ix, ixp1, j, mrk, mrkj, m1, m2;
   int *marks;
   double *x, *y;
   double u, v, lg;
   double d2, a, cifval;
+  MultiStrauss *multistrauss;
+
+  multistrauss = (MultiStrauss *) cdata;
 
   u  = prop.u;
   v  = prop.v;
@@ -110,12 +118,12 @@ double straussmcif(prop, state)
   Rprintf("computing cif: u=%lf, v=%lf, mrk=%d\n", u, v, mrk);
 #endif
 
-  cifval = MultiStrauss.beta[mrk];
+  cifval = multistrauss->beta[mrk];
 
   if(npts == 0) 
     return(cifval);
 
-  ntypes = MultiStrauss.ntypes;
+  ntypes = multistrauss->ntypes;
 
 #ifdef DEBUG
   Rprintf("initialising pair counts\n");
@@ -124,7 +132,7 @@ double straussmcif(prop, state)
   /* initialise pair counts */
   for(m1 = 0; m1 < ntypes; m1++)
     for(m2 = 0; m2 < ntypes; m2++)
-      MAT(MultiStrauss.kount, m1, m2, ntypes) = 0;
+      MAT(multistrauss->kount, m1, m2, ntypes) = 0;
 
   /* compile pair counts */
 
@@ -134,21 +142,21 @@ double straussmcif(prop, state)
 
   ixp1 = ix+1;
   /* If ix = NONE = -1, then ixp1 = 0 is correct */
-  if(MultiStrauss.per) { /* periodic distance */
+  if(multistrauss->per) { /* periodic distance */
     if(ix > 0) {
       for(j=0; j < ix; j++) {
 	mrkj = marks[j];
-	d2 = dist2(u,v,x[j],y[j],MultiStrauss.period);
-	if(d2 < MAT(MultiStrauss.rad2, mrk, mrkj, ntypes)) 
-	  MAT(MultiStrauss.kount, mrk, mrkj, ntypes)++;
+	d2 = dist2(u,v,x[j],y[j],multistrauss->period);
+	if(d2 < MAT(multistrauss->rad2, mrk, mrkj, ntypes)) 
+	  MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
       }
     }
     if(ixp1 < npts) {
       for(j=ixp1; j<npts; j++) {
 	mrkj = marks[j];
-	d2 = dist2(u,v,x[j],y[j],MultiStrauss.period);
-	if(d2 < MAT(MultiStrauss.rad2, mrk, mrkj, ntypes)) 
-	  MAT(MultiStrauss.kount, mrk, mrkj, ntypes)++;
+	d2 = dist2(u,v,x[j],y[j],multistrauss->period);
+	if(d2 < MAT(multistrauss->rad2, mrk, mrkj, ntypes)) 
+	  MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
       }
     }
   }
@@ -156,22 +164,22 @@ double straussmcif(prop, state)
     if(ix > 0) {
       for(j=0; j < ix; j++) {
 	mrkj = marks[j];
-	a = MAT(MultiStrauss.rad2, mrk, mrkj, ntypes); 
+	a = MAT(multistrauss->rad2, mrk, mrkj, ntypes); 
 	a -= pow(u - x[j], 2);
 	if(a > 0) {
 	  a -= pow(v - y[j], 2);
-	  if(a > 0) MAT(MultiStrauss.kount, mrk, mrkj, ntypes)++;
+	  if(a > 0) MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
 	}
       }
     }
     if(ixp1 < npts) {
       for(j=ixp1; j<npts; j++) {
 	mrkj = marks[j];
-	a = MAT(MultiStrauss.rad2, mrk, mrkj, ntypes); 
+	a = MAT(multistrauss->rad2, mrk, mrkj, ntypes); 
 	a -= pow(u - x[j], 2);
 	if(a > 0) {
 	  a -= pow(v - y[j], 2);
-	  if(a > 0) MAT(MultiStrauss.kount, mrk, mrkj, ntypes)++;
+	  if(a > 0) MAT(multistrauss->kount, mrk, mrkj, ntypes)++;
 	}
       }
     }
@@ -183,14 +191,14 @@ double straussmcif(prop, state)
   /* multiply cif value by pair potential */
   for(m1 = 0; m1 < ntypes; m1++) {
     for(m2 = 0; m2 < ntypes; m2++) {
-      kount = MAT(MultiStrauss.kount, m1, m2, ntypes);
-      if(MAT(MultiStrauss.hard, m1, m2, ntypes)) {
+      kount = MAT(multistrauss->kount, m1, m2, ntypes);
+      if(MAT(multistrauss->hard, m1, m2, ntypes)) {
 	if(kount > 0) {
 	  cifval = 0.0;
 	  return(cifval);
 	}
       } else {
-	lg = MAT(MultiStrauss.loggamma, m1, m2, ntypes);
+	lg = MAT(multistrauss->loggamma, m1, m2, ntypes);
 	cifval *= exp(lg * kount);
       }
     }
